@@ -121,12 +121,19 @@ NAME is the name of the widget.
 TYPE is the type of widget.  You can specify :archive, :category,
 :branch, or :version."
   (let* ((args (widget-get parent :args))
+         (largs (length args))
          (index (tla--position name args (lambda (e w)
                                            (let ((node (widget-get w :node)))
                                              ;; Next line is hack for version node.
                                              (unless node (setq node w))
                                              (string= e (widget-get node type))))))
-         (tree (when index (nth index (widget-get parent :children))))
+         (children (widget-get parent :children))
+         (lchildren (length children))
+         ;; The internal data structure of tree-widget bundled to develoment
+         ;; version of GNU Emacs may by changed; :children list becomes longer
+         ;; than :args list.
+         (tree (when index (nth (+ index (if (eq largs lchildren) 0 1))
+                                children)))
          (node (when tree (save-excursion (goto-char (widget-get tree :from))
                                           (goto-char (next-single-property-change (point) 'widget))
                                           (tla--widget-node-get-at)))))
@@ -265,12 +272,20 @@ WIDGET must be of type tla--widget-*-node."
   :menu nil
   :marks " "
   :keep '(:marks :open)
-  :open-subtree (if (fboundp 'tree-widget-open-node)
-		    'tree-widget-open-node
-		  'tla--tree-widget-node-toggle-subtree-for-tree-widget-v1)
-  :close-subtree (if (fboundp 'tree-widget-open-node)
-		     'tree-widget-close-node
-		   'tla--tree-widget-node-toggle-subtree-for-tree-widget-v1))
+  :open-subtree (cond
+                 ((fboundp 'tree-widget-action)
+                  'tla--tree-widget-node-open-subtree-for-tree-widget-action)
+                 ((fboundp 'tree-widget-open-node)
+                  'tree-widget-open-node)
+                 (t
+		  'tla--tree-widget-node-toggle-subtree-for-tree-widget-v1))
+  :close-subtree (cond
+                  ((fboundp 'tree-widget-action)
+                   'tla--tree-widget-node-close-subtree-for-tree-widget-action)
+                  ((fboundp 'tree-widget-open-node)
+                   'tree-widget-close-node)
+                  (t
+                   'tla--tree-widget-node-toggle-subtree-for-tree-widget-v1)))
 
 (defvar tla--widget-node-map
   (let ((map (copy-keymap tla--context-map-template)))
@@ -355,10 +370,25 @@ in the archives tree."
                                            branch)
   "Refresh node and LEVEL subnode at the POINT.
 Before refreshing node, names cache are also refreshed if
-ARCHIVE, CATEGORY, and/or BRANCH are specified."
+ARCHIVE, CATEGORY, and/or BRANCH are specified.
+If POINT is a symbol, `name', node is specified by ARCHIVE,
+CATEGORY, and/or BRANCH."
   (interactive)
   (unless level (setq level 1))
-  (unless point (setq point (point)))
+  (setq point (cond
+               ((null point) (point))
+               ((eq 'name point)
+                (save-excursion
+                  (goto-char
+                   (next-single-property-change
+                    (widget-get
+                     (tla--browse-find-single-widget
+                      archive
+                      category
+                      branch)
+                     :from)
+                    'widget))))
+               (t point)))
   (if branch
       (tla--archive-tree-build-versions archive
                                         category
@@ -373,6 +403,7 @@ ARCHIVE, CATEGORY, and/or BRANCH are specified."
                                               nil
                                               t)
         (tla--archive-tree-build-archives nil t))))
+
   (let* ((widget (tla--widget-node-get-at point))
          (tree (tla--widget-get-ancestor widget level)))
     (widget-put tree :args nil)
@@ -493,6 +524,20 @@ by FORCE."
           (tla--widget-node-toggle-subtree-internal
            full-widget force t))))))
 
+(defun tla--tree-widget-node-open-subtree-for-tree-widget-action (widget)
+  "Open tree node function used in `tla-browse' with tree-widget bundled to
+development version of GNU Emacs."
+  (let ((parent (widget-get widget :parent)))
+    (unless (widget-get parent :open)
+      (tree-widget-action parent))))
+
+(defun tla--tree-widget-node-close-subtree-for-tree-widget-action (widget)
+  "Close tree node function used in `tla-browse' with tree-widget bundled to
+development version of GNU Emacs."
+  (let ((parent (widget-get widget :parent)))
+    (when (widget-get parent :open)
+      (tree-widget-action parent))))
+
 (defun tla--tree-widget-node-toggle-subtree-for-tree-widget-v1 (widget)
   "Toggle tree node function used in `tla-browse' with tree-widget ver.1.0.5.
 The code is the almost same as in tree-widget-toggle-folding tree-widget version
@@ -729,9 +774,8 @@ This may be useful to search an archive named NAME."
                                                 archive))))))
              (widget-put (widget-get res :node) :parent res)
              res))
-         (let* ((l tla--archive-tree))
-           (when (or (null l) current-prefix-arg)
-             (tla--archive-tree-build-archives nil t))
+         (progn
+           (tla--archive-tree-build-archives (not current-prefix-arg) t)
            tla--archive-tree)))))
 
 (defun tla--widget-archive-node-value-create (widget)
@@ -865,7 +909,11 @@ Unmark widgets not associated with the default archive.
       (error "No archive under the point"))
     (tla-mirror-archive archive nil nil nil nil)
     (tla--widget-node-refresh 2)
-    (tla--widget-archives-root-node-goto (format "%s-MIRROR" archive))
+    (tla--widget-archives-root-node-goto (format
+                                          (if (tla-use-baz-archive-registration)
+                                              "%s"
+                                            "%s-MIRROR")
+                                          archive))
     (tla--flash-line)))
 
 (defun tla--widget-archive-node-make-mirror-at-local ()
@@ -1115,7 +1163,10 @@ Unmark widgets not associated with the default archive.
       'tla--widget-version-node-show-revisions)
     (define-key map tla--keyvec-get
       'tla--widget-version-node-get-version)
-    (define-key map tla--keyvec-tag 'tla--widget-version-node-tag)
+    (define-key map tla--keyvec-tag
+      'tla--widget-version-node-tag)
+    (define-key map [?L]
+      'tla--widget-version-node-add-to-library)
     map)
   "Keymap used on tla--widget-version-node.")
 
@@ -1125,6 +1176,7 @@ Unmark widgets not associated with the default archive.
     ["Show Revisions" tla--widget-version-node-show-revisions t]
     ["Remove Version Registration[NOT IMPLEMENTED]" nil t]
     ["Get..." tla--widget-version-node-get-version t]
+    ["Add to Library" tla--widget-version-node-add-to-library t]
     ["Add a Bookmark" tla--widget-node-add-bookmark t]
     ["Synchronize Mirror to Remote"
      tla--widget-node-synchronize-mirror-to-remote t]
@@ -1230,13 +1282,25 @@ If POINT is nil, use the point under `point'."
 		 version)
       (error "No version under the point"))))
 
+(defun tla--widget-version-node-add-to-library ()
+  "Run \"tla library-add\" against the version at point."
+  (interactive)
+  (let* ((name (tla--widget-node-get-name))
+	 (archive (tla--name-archive name))
+	 (category (tla--name-category name))
+	 (branch (tla--name-branch name))
+	 (version (tla--name-version name)))
+    (if version
+        (tla-library-add archive category branch version)
+      (error "No version under the point"))))
+
 (defun tla--widget-version-node-tag ()
   "Run tla tag from the version under the point."
   (interactive)
   (let* ((from (tla--widget-node-get-name))
 	 (from-fq (tla--name-construct from))
 	 (to   (tla-name-read (format "Tag from `%s' to: " from-fq)
-			       'prompt 'prompt 'prompt 'prompt))
+                              'prompt 'prompt 'prompt 'prompt))
 	 (to-fq (tla--name-construct to)))
     (unless from
       (error "No version under the point"))
@@ -1244,10 +1308,29 @@ If POINT is nil, use the point under `point'."
       (error "Wrong version tagged to is given"))
     (save-excursion
       (tla--version-tag-internal from-fq to-fq 'synchronously))
-    (tla--widget-node-refresh 1 nil
-			      (tla--name-archive to-fq)
-			      (tla--name-category to-fq)
-			      (tla--name-branch to-fq))
+    ;;
+    (tla--browse-open nil
+                      (tla--name-archive to-fq))
+    (tla--widget-node-refresh 1
+                              'name
+                              (tla--name-archive to-fq))
+    (tla--browse-open nil
+                      (tla--name-archive to-fq)
+                      (tla--name-category to-fq))
+
+    (tla--widget-node-refresh 1
+                              'name
+                              (tla--name-archive to-fq)
+                              (tla--name-category to-fq))
+    (tla--browse-open nil
+                      (tla--name-archive to-fq)
+                      (tla--name-category to-fq)
+                      (tla--name-branch to-fq))
+    (tla--widget-node-refresh 1
+                              'name
+                              (tla--name-archive to-fq)
+                              (tla--name-category to-fq)
+                              (tla--name-branch to-fq))
     (tla--browse-open t
                       (tla--name-archive to-fq)
                       (tla--name-category to-fq)
