@@ -113,6 +113,7 @@ Must be a subdir of `tla-tests-scratch-dir'.")
     (tla-test-revision-lessp noid noarch noproject)
     (tla-test-recursive-update noproject nocmdcheck)
     (tla-test--position)
+    (tla-test--digit-char-p)
     )
   "Alist used by the initialization phase of each test.
 
@@ -173,7 +174,7 @@ Log messages are written to the tests log buffer."
 
 (defmacro tla-write-to-file (filename &rest forms)
   "In buffer visiting FILENAME, evaluate FORMS, save and kill the buffer."
-  (let ((buf (gensym)))
+  (let ((buf (tla--gensym)))
     `(let ((,buf (find-file-noselect ,filename)))
        (unwind-protect
            (with-current-buffer ,buf
@@ -192,7 +193,7 @@ Log messages are written to the tests log buffer."
                              (format "%s/%s"
                                      tla-tests-archive-name
                                      tla-tests-project-name)))
-    (tla--run-tla-sync (list "import" (when (tla-import-needs-setup) "--setup")))
+    (tla--run-tla-sync (list "import" (when (tla-import-has-setup-option) "--setup")))
     (tla-write-to-file "hello" (insert (concat "Current time is "
                                                (current-time-string))))
     (tla-add nil "hello")
@@ -514,6 +515,23 @@ a difference."
         (eq nil (tla--position 4.0 list (lambda (x y) (= x y))))
       (error "Wrong position"))))
 
+
+(defun tla-test--digit-char-p ()
+  "Test `tla--digit-char-p'."
+  (when (member nil
+          (list
+           (tla--digit-char-p ?5)
+           (tla--digit-char-p ?9)
+           (tla--digit-char-p ?0)
+           (tla--digit-char-p ?1)
+
+           (not (tla--digit-char-p ?a))
+           (not (tla--digit-char-p ?A))
+           (not (tla--digit-char-p ?!))
+           (not (tla--digit-char-p ?Y))))
+    (error "Failed")))
+
+
 (provide 'xtla-tests)
 ;; arch-tag: 4fce7821-79bb-449e-aeba-221f8ac65cc3
 ;;; xtla-tests.el ends here
