@@ -15,7 +15,9 @@ while test $# -ne 0; do
 done
 
 
-dpkg-buildpackage -rfakeroot
+echo "building package ..."
+dpkg-buildpackage -rfakeroot -d
+echo "building package ... done"
 
 mkdir -p ++apt-repository
 cd ++apt-repository
